using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using Dapfor.Net.Diagnostics;
using Dapfor.Net.Ui;
using Timer=System.Windows.Forms.Timer;

namespace Dapfor.Net.Grid.Benchmark.DataUpdating
{
    public partial class DataUpdatingNonEventModelControl : UserControl
    {
        readonly Random random = new Random();
        private bool measuring;
        private DateTime measureStart;
        private int updatesCount;
        private readonly List<int> updateScenarios = new List<int>();
        PerformanceCounter cpuCounter = new PerformanceCounter();
        
        private DateTime lastStat = DateTime.UtcNow;
        private int lastStatCount;

        
        private Timer timer;

        public DataUpdatingNonEventModelControl()
        {
            InitializeComponent();

            //Initialize performance counters
            cpuCounter.CategoryName = "Processor";
            cpuCounter.CounterName = "% Processor Time";
            cpuCounter.InstanceName = "_Total";

            grid.Highlighting.Fading = false;


            //Initialize header and columns
            grid.Headers.Add(new Header());
            for (int i = 0; i < 10; i++)
            {
                grid.Headers[0].Add(new Column("id" + i, "Value"+ i, 80));
            }

            grid.PaintRow += delegate(object sender, PaintRowEventArgs e)
            {
                e.Parts &= e.Parts ^ PaintPart.RowSelectorImages;

                e.PaintAll();
                e.Handled = true;

                using (StringFormat sf = new StringFormat())
                {
                    sf.Alignment = StringAlignment.Far;
                    e.Graphics.DrawString(e.Row.VisibleIndex.ToString(), SystemFonts.CaptionFont, SystemBrushes.InfoText, e.RowSelectorBounds, sf);
                }
            };
            
            grid.ProcessUpdate += OnUpdateCell;

            grid.PaintRectangle += delegate(object sender, PaintRectangleEventArgs e)
            {
                if (grid.Nodes.Count == 0)
                {
                    e.PaintAll();
                    e.Handled = true;

                    //Draw text and logo if there are no rows
                    Point pt = new Point(e.Grid.Width - Properties.Resources.logo.Width, e.Grid.Height - Properties.Resources.logo.Height);
                    e.Graphics.DrawImage(Properties.Resources.logo, new Rectangle(pt, Properties.Resources.logo.Size));

                    Rectangle rc = grid.ClientRectangle;
                    rc.Width -= Properties.Resources.logo.Width;
                    rc.Offset(10, 30);

                    using (Font f = new Font("verdana", 13, GraphicsUnit.Pixel))
                    using (Brush b = new SolidBrush(Color.FromArgb(0x55, 0x66, 0x55)))
                    using (StringFormat sf = new StringFormat())
                    {
                        sf.Alignment = StringAlignment.Near;
                        sf.LineAlignment = StringAlignment.Near;
                        sf.FormatFlags &= StringFormatFlags.NoWrap ^ sf.FormatFlags;
                        e.Graphics.DrawString("Lets say some general things about application performance. A running application consumes resources to recalculate and repaint data using GDI/GDI+ tools. Both operations are not linked with each other. Under recalculation we mean sorting, filtering, grouping and all other operations except repainting. All these operations in a grid are performed only in GUI thread. If data changes in another thread, the grid performs the required synchronization, usually synchronous data transfer to GUI thread. Event-driven model may feature asynchronous transfer of notifications when the calling thread is not blocked. Both data recalculation and repainting consume CPU resources in different proportions with different number of rows. If there are a few rows, repainting is the most resource-intensive task because most rows are contained in the visible part of the grid that requires intense repainting. When there are more rows, only few of them are in the visible area, therefore, repainting consumes less resources.\r\n\r\nIts worth mentioning that fading effect, when every cell is repainted every 30 ms with color change, has a negative impact on performance and should not be used in real-time applications.", f, b, rc, sf);
                    }
                }
            };
        }

        public void StopTest()
        {
            grid.StopLoadBalancingProcess();
            updatesCount = 0;
            measuring = true;
            measureStart = DateTime.UtcNow;
            lastStatCount = 0;
            lastStat = DateTime.UtcNow;
            gbStatistics.Visible = false;
            grid.Rows.Clear();
            btnStart.Enabled = true;
            cbEnableSort.Enabled = true;
            cbFading.Enabled = true;
            cbNonGuiThread.Enabled = true;
        }


        void StartTest()
        {
            if(updateScenarios.Count > 0)
            {
                grid.Highlighting.Fading = cbFading.Checked;

                int rowsCount = updateScenarios[0];
                updateScenarios.RemoveAt(0);

                grid.StopLoadBalancingProcess();

                grid.Headers[0].SortedColumns.Clear();


                //Populate the grid
                grid.Rows.Clear();
                for (int i = 0; i < rowsCount; ++i)
                {
                    grid.Rows.Add(new int[]{random.Next(), random.Next(),random.Next(),random.Next(),random.Next(),random.Next(),random.Next(),random.Next(),random.Next(),random.Next()});
                }

                if(cbEnableSort.Checked)
                {
                    grid.Headers[0][1].SortDirection = SortDirection.Ascending;
                    grid.Headers[0][3].SortDirection = SortDirection.Descending;
                }

                measuring = false;
                btnStart.Enabled = false;
                cbEnableSort.Enabled = false;
                cbFading.Enabled = false;
                cbNonGuiThread.Enabled = false;

                grid.StartLoadBalancingProcess();
            }
            else
            {
                measuring = false;
                btnStart.Enabled = true;
                cbEnableSort.Enabled = true;
                cbFading.Enabled = true;
                cbNonGuiThread.Enabled = true;

                grid.StopLoadBalancingProcess();

                Logger.Info(string.Format("Measurement completed"));
            }
        }



        private void OnUpdateCell(object sender, EventArgs e)
        {
            if(grid.IsLimitFound && !measuring)
            {
                updatesCount = 0;
                measuring = true;
                measureStart = DateTime.UtcNow;
                lastStatCount = 0;
                lastStat = DateTime.UtcNow;
            }


            //Updates random cell in the grid;
            Row row = grid.Rows.Count > 0 ? grid.Rows[random.Next() % grid.Rows.Count] : null;
            Cell cell = row != null ? row[random.Next() % grid.Headers[0].Count] : null;

            if (cell != null)
            {
                int value = random.Next();
                if(cbNonGuiThread.Checked)
                {
                    ThreadPool.QueueUserWorkItem(delegate
                    {
                        cell.Value = value;
                    });    
                }
                else
                {
                    cell.Value = value;
                }
            }

            updatesCount++;
            if(measuring)
            {
                //Measure refresh rate for 5 seconds
                if(DateTime.UtcNow - measureStart > TimeSpan.FromSeconds(5))
                {
                    grid.StopLoadBalancingProcess();

                    int visibleRows = grid.Rows.BottomVisibleRow != null && grid.Rows.TopVisibleRow != null
                        ? (grid.Rows.BottomVisibleRow.VisibleIndex - grid.Rows.TopVisibleRow.VisibleIndex)
                        : 0;
                    long visibleCells = grid.Headers[0].Count * visibleRows;
                    long totalCells = grid.Headers[0].Count * grid.Rows.Count;

                    double rate = totalCells != 0 ? (double)visibleCells / totalCells : 0;


                    Logger.Info(string.Format("Grid of {0} rows, size = {2}x{3}. Maximum refresh rate: {1} updates/sec, in visible bounds: {4:0} updates/sec", 
                                              grid.Rows.Count, 
                                              updatesCount / 5, 
                                              grid.ClientSize.Width,
                                              grid.ClientSize.Height,
                                              rate * updatesCount / 5));

                    //Execute the next scenario
                    StartTest();
                }
            }

            DisplayStatistics();
        }

        private void DisplayStatistics()
        {
            if(DateTime.UtcNow - lastStat > TimeSpan.FromMilliseconds(500))
            {
                lbCPULoadRate.Text = string.Format("CPU load rate: {0:0} %", cpuCounter.NextValue());
                lbGridSize.Text = string.Format("Grid size: {0}x{1}", grid.ClientSize.Width, grid.ClientSize.Height);
                double updates = 1000 * (updatesCount - lastStatCount) / (DateTime.UtcNow - lastStat).TotalMilliseconds;
                lbUpdateRate.Text = string.Format("Update rate: {0:0} updates/sec", updates);

                int visibleRows = grid.Rows.BottomVisibleRow != null && grid.Rows.TopVisibleRow != null
                        ? (grid.Rows.BottomVisibleRow.VisibleIndex - grid.Rows.TopVisibleRow.VisibleIndex)
                        : 0;
                long visibleCells = grid.Headers[0].Count * visibleRows;
                long totalCells = grid.Headers[0].Count * grid.Rows.Count;
                
                double rate = totalCells != 0 ? (double)visibleCells / totalCells : 0;
                lbVisibleBoundsUpdateRate.Text = string.Format("Update rate in visible bounds: {0:0} updates/sec", rate * updates);

                gbStatistics.Visible = true;

                lastStatCount = updatesCount;
                lastStat = DateTime.UtcNow;
            }
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            updateScenarios.Add(10);
            updateScenarios.Add(20);
            updateScenarios.Add(50);
            updateScenarios.Add(100);
            updateScenarios.Add(200);
            updateScenarios.Add(500);
            updateScenarios.Add(1000);
            updateScenarios.Add(5000);
            updateScenarios.Add(10000);
            updateScenarios.Add(20000);
            updateScenarios.Add(50000);
            updateScenarios.Add(100000);

            Logger.Info(string.Format("Application measures performance of .Net Grid. It may take a little time..."));

            StartTest();
        }
    }
}