using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using Dapfor.Net.Diagnostics;
using Dapfor.Net.Ui;
using Timer=System.Windows.Forms.Timer;

namespace Dapfor.Net.Grid.Benchmark.Grouping
{
    public partial class DataGroupingControl : UserControl
    {
        private const int groupCount = 20;
        readonly Random random = new Random();
        private bool measuring;
        private DateTime measureStart;
        private int updatesCount;
        private int rowsCount;
        private readonly List<int> updateScenarios = new List<int>();
        PerformanceCounter cpuCounter = new PerformanceCounter();
        static readonly HighPerformanceTimer hiPerfTimer = new HighPerformanceTimer();
        
        private DateTime lastStat = DateTime.UtcNow;
        private int lastStatCount;

        
        private Timer timer;

        public DataGroupingControl()
        {
            InitializeComponent();

            //Initialize performance counters
            cpuCounter.CategoryName = "Processor";
            cpuCounter.CounterName = "% Processor Time";
            cpuCounter.InstanceName = "_Total";

            grid.Highlighting.Fading = false;


            //Initialize header and columns
            grid.Headers.Add(Header.FromDataType(typeof (DataObject)));
            foreach (Column column in grid.Headers[0])
            {
                column.Width = 80;
                column.Sortable = false;
            }
            grid.Headers[0].GroupingEnabled = true;
            grid.Headers[0][0].Grouped = true;
            grid.Headers[0][0].Visible = false;

            
            grid.ProcessUpdate += OnUpdateCell;

            grid.PaintRectangle += delegate(object sender, PaintRectangleEventArgs e)
            {
                if (grid.Nodes.Count == 0)
                {
                    e.PaintAll();
                    e.Handled = true;

                    //Draw text and logo if there are no rows
                    Point pt = new Point(e.Grid.Width - Properties.Resources.logo.Width, e.Grid.Height - Properties.Resources.logo.Height);
                    e.Graphics.DrawImage(Properties.Resources.logo, new Rectangle(pt, Properties.Resources.logo.Size));

                    Rectangle rc = grid.ClientRectangle;
                    rc.Width -= Properties.Resources.logo.Width;
                    rc.Offset(10, 50);

                    using (Font f = new Font("verdana", 13, GraphicsUnit.Pixel))
                    using (Brush b = new SolidBrush(Color.FromArgb(0x55, 0x66, 0x55)))
                    using (StringFormat sf = new StringFormat())
                    {
                        sf.Alignment = StringAlignment.Near;
                        sf.LineAlignment = StringAlignment.Near;
                        sf.FormatFlags &= StringFormatFlags.NoWrap ^ sf.FormatFlags;
                        e.Graphics.DrawString("Application measures .NetGrid characteristics for static and real-time data grouping. During static grouping .NetGrid rebuilds content for all data, while dependence of grouping rate on data volume is measured. During dynamic grouping data is changed in real-time, and this requires searching for existing groups, creating new groups and removing groups that contain no more rows.\r\n\r\nTo understand the performance for dynamic data regrouping, remember that when there are a few rows, they have to be repainted frequently, and when there are many rows, more resources are consumed for searching or creating groups.", f, b, rc, sf);
                    }
                }
            };
        }

        public void StopTest()
        {
            grid.StopLoadBalancingProcess();
            updatesCount = 0;
            measuring = true;
            measureStart = DateTime.UtcNow;
            lastStatCount = 0;
            lastStat = DateTime.UtcNow;
            gbStatistics.Visible = false;
            grid.Rows.Clear();
            btnStartRegrouping.Enabled = true;
            btnGrouping.Enabled = true;
        }


        void StartTest()
        {
            if(updateScenarios.Count > 0)
            {
                rowsCount = updateScenarios[0];
                updateScenarios.RemoveAt(0);

                grid.StopLoadBalancingProcess();

                grid.Headers[0].SortedColumns.Clear();


                //Populate the grid
                grid.Rows.Clear();
                for (int i = 0; i < rowsCount; ++i)
                {
                    //Limit a number of groups, otherwise the grid will have a group per row
                    DataObject dataObject = new DataObject();
                    //dataObject.Value0 = random.Next() % groupCount;
                    int groups = Math.Min(rowsCount, groupCount);

                    dataObject.Value0 = i / (rowsCount / groups);

                    grid.Rows.Add(dataObject);
                }

                measuring = false;
                btnStartRegrouping.Enabled = false;
                btnGrouping.Enabled = false;
                grid.StartLoadBalancingProcess();
            }
            else
            {
                measuring = false;
                btnStartRegrouping.Enabled = true;
                btnGrouping.Enabled = true;
                grid.StopLoadBalancingProcess();

                Logger.Info(string.Format("Measurement completed"));
            }
        }



        private void OnUpdateCell(object sender, EventArgs e)
        {
            if(grid.IsLimitFound && !measuring)
            {
                updatesCount = 0;
                measuring = true;
                measureStart = DateTime.UtcNow;
                lastStatCount = 0;
                lastStat = DateTime.UtcNow;
            }


            //Updates random cell in the grid;
            Row row = grid.Rows.Count > 0 ? grid.Rows[random.Next() % grid.Rows.Count] : null;
            Cell cell = row != null ? row[random.Next() % grid.Headers[0].Count] : null;

            if (cell != null)
            {
                int value = random.Next();
                
                //Limit a number of groups    
                if (cell.Column.Id == "Value0")
                {
                    value = random.Next() % groupCount; 
                }
                
                if(cbNonGuiThread.Checked)
                {
                    ThreadPool.QueueUserWorkItem(delegate
                    {
                        cell.Value = value;
                    });    
                }
                else
                {
                    cell.Value = value;
                }
            }

            updatesCount++;
            if(measuring)
            {
                //Measure refresh rate for 5 seconds
                if(DateTime.UtcNow - measureStart > TimeSpan.FromSeconds(5))
                {
                    grid.StopLoadBalancingProcess();

                    int visibleRows = grid.Rows.BottomVisibleRow != null && grid.Rows.TopVisibleRow != null
                        ? (grid.Rows.BottomVisibleRow.VisibleIndex - grid.Rows.TopVisibleRow.VisibleIndex)
                        : 0;
                    long visibleCells = grid.Headers[0].Count * visibleRows;
                    long totalCells = grid.Headers[0].Count * rowsCount;

                    double rate = totalCells != 0 ? (double)visibleCells / totalCells : 0;


                    Logger.Info(string.Format("Grid of {0} rows, size = {2}x{3}. Maximum regrouping rate: {1} updates/sec, in visible bounds: {4:0} updates/sec",
                                              rowsCount, 
                                              updatesCount / 5, 
                                              grid.ClientSize.Width,
                                              grid.ClientSize.Height,
                                              rate * updatesCount / 5));

                    //Execute the next scenario
                    StartTest();
                }
            }

            DisplayStatistics();
        }

        private void DisplayStatistics()
        {
            if(DateTime.UtcNow - lastStat > TimeSpan.FromMilliseconds(500))
            {
                lbCPULoadRate.Text = String.Format("CPU load rate: {0:0} %", cpuCounter.NextValue());
                lbGridSize.Text = string.Format("Grid size: {0}x{1}", grid.ClientSize.Width, grid.ClientSize.Height);
                double updates = 1000 * (updatesCount - lastStatCount) / (DateTime.UtcNow - lastStat).TotalMilliseconds;
                lbUpdateRate.Text = string.Format("Update rate: {0:0} updates/sec", updates);

                int visibleRows = grid.Rows.BottomVisibleRow != null && grid.Rows.TopVisibleRow != null
                        ? (grid.Rows.BottomVisibleRow.VisibleIndex - grid.Rows.TopVisibleRow.VisibleIndex)
                        : 0;

                long visibleCells = grid.Headers[0].Count * visibleRows;
                long totalCells = grid.Headers[0].Count * grid.Rows.Count;
                
                double rate = totalCells != 0 ? (double)visibleCells / totalCells : 0;
                lbVisibleBoundsUpdateRate.Text = string.Format("Update rate in visible bounds: {0:0} updates/sec", rate * updates);

                gbStatistics.Visible = true;

                lastStatCount = updatesCount;
                lastStat = DateTime.UtcNow;
            }
        }

        private void OnStartRegroupingTest(object sender, EventArgs e)
        {
            updateScenarios.Add(10);
            updateScenarios.Add(20);
            updateScenarios.Add(50);
            updateScenarios.Add(100);
            updateScenarios.Add(200);
            updateScenarios.Add(500);
            updateScenarios.Add(1000);
            updateScenarios.Add(5000);
            updateScenarios.Add(10000);
            updateScenarios.Add(20000);
            updateScenarios.Add(50000);
            updateScenarios.Add(100000);

            Logger.Info(string.Format("Application measures performance of .Net Grid while real-time regrouping. It may take a little time..."));

            StartTest();
        }

        private void OnStartGroupingTest(object sender, EventArgs e)
        {
            btnStartRegrouping.Enabled = false;
            btnGrouping.Enabled = false;

            
            const int tranche = 10000;

            Logger.Info(string.Format("Performance measurement when .Net Grid groups rows."));
            Application.DoEvents();

            //10 iterations
            for (int iteration = 0; iteration < 10; ++iteration)
            {
                grid.Headers[0].SortedColumns.Clear();
                grid.Rows.Clear();
                grid.Headers[0].GroupedColumns.Clear();


                //Add rows to the grid non-sorted and non-grouped grid
                for (int i = 0; i < (iteration + 1) * tranche; ++i)
                {
                    //Limit a number of groups, otherwise the grid will have a group per row
                    DataObject dataObject = new DataObject();
                    dataObject.Value0 = random.Next() % groupCount;
                    grid.Rows.Add(dataObject);
                }

                hiPerfTimer.Start();

                //Group by the first column
                grid.Headers[0]["Value0"].Grouped = true;
                grid.Headers[0]["Value0"].Visible = false;

                hiPerfTimer.Stop();

                int rate = hiPerfTimer.Duration > 0 ? (int)(1000 * ((iteration + 1) * tranche) / hiPerfTimer.Duration) : 0;
                Logger.Info(string.Format("Grid grouped {0} rows in {1:0} msec. Rate: {2} rows/sec", (iteration + 1)* tranche, hiPerfTimer.Duration, rate));
                Application.DoEvents();
            }


            btnStartRegrouping.Enabled = true;
            btnGrouping.Enabled = true;
            Logger.Info(string.Format("Measurement completed"));
        }
    }
}