using System;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace Dapfor.Net.Grid.Benchmark
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            //This code enables ImageList to contain 32-bit images. (Unfortunately Designer is unable to create ImageList with 32-bit semi-transparent images)
            ImageList il = new ImageList();
            il.ColorDepth = ColorDepth.Depth32Bit;
            il.Images.Add(Properties.Resources.about);
            tabDataUpdating.ImageList = il;
            tabNetGridPerformance.ImageIndex = 0;

            BuildHtmlContent();

//->****
            logViewer1.DisplayInspector = false;
            logViewer1.DisplayToolbar = true;
            logViewer1.InspectingStyle = Diagnostics.InspectingStyle.Hierarchical;
            logViewer1.MaxRowCount = 0;
//<-****
        }

        private void BuildHtmlContent()
        {
            //Prepare images to display them in webbrowser
            string path = Path.Combine(Path.GetTempPath(), "Dapfor.Benchmark");

            //Ensure directory
            if (!Directory.Exists(path))
            {
                Directory.CreateDirectory(path);
            }

            //Copy images
            string basePath = typeof(Program).Namespace + ".HtmlContent";
            foreach (string resource in typeof(Program).Assembly.GetManifestResourceNames())
            {
                if (resource.Contains(basePath))
                {
                    using (FileStream fs = new FileStream(Path.Combine(path, resource), FileMode.OpenOrCreate))
                    using (BinaryWriter bw = new BinaryWriter(fs))
                    using (Stream stream = typeof(Program).Assembly.GetManifestResourceStream(resource))
                    {
                        if (stream != null)
                        {
                            byte[] bytes = new byte[stream.Length];
                            stream.Read(bytes, 0, bytes.Length);
                            bw.Write(bytes);
                            bw.Flush();
                        }
                    }
                }
            }

            
            string prefix = !string.IsNullOrEmpty(path) && !string.IsNullOrEmpty(basePath) ? Path.Combine(path, basePath + ".") : string.Empty;


            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("<html><head> <link href=\"{0}style.css\" rel=\"stylesheet\" type=\"text/css\" /></head><body>", prefix);
            sb.Append("<h2>Dapfor .Net Grid performance</h2>");
            sb.Append("<p>When a programmer develops an application, sooner or later he has to deal with its performance. To ensure application performance he has to think over its architecture, ");
            sb.Append("pick necessary components and use them correctly. Otherwise, on late stages of development, when he has a lot of code, improving performance becomes a complex task ");
            sb.Append("requiring changes of working and debugged code.</p>");
            sb.AppendFormat("<img src=\"{0}Performance.jpg\"/>", prefix);
            sb.Append("<p>In this section we describe features of Dapfor grid that help to evaluate its performance and to understand, which modes should be used for specific applications.</p>");
            sb.Append("<p>Test configuration: <ul><li>Intel Core 2 Quad Q6600 2.40 GHz, Memory : 2 GB, OS Windows Vista SP2, display resolution: 19201280</li><li>Tested grid dimensions: 1890813. During all tests the grid is always visible on the screen.</li></ul></p>");
            sb.Append("<p>Test results may vary for different computers or when visible grid dimensions are change.</p>");
            
            sb.Append("<p>Lets say some general things about application performance. A running application consumes resources to recalculate and repaint data using GDI/GDI+ tools. Both operations are not linked with each other. Under recalculation we mean ");
            sb.Append("sorting, filtering, grouping and all other operations except repainting. All these operations in a grid are performed only in GUI thread. If data changes ");
            sb.Append("in another thread, the grid performs the required synchronization, usually synchronous data transfer to GUI thread. Event-driven may feature asynchronous transfer ");
            sb.Append("of notifications when the calling thread is not blocked. Both data recalculation and repainting consume CPU resources in different proportions with different number of rows. ");
            sb.Append("If there are a few rows, repainting is the most resource-intensive task because most rows are contained in the visible part of the grid that requires intense repainting. ");
            sb.Append("When there are more rows, only few of them are in the visible area, therefore, repainting consumes less resources.</p>");

            sb.Append("<p>Lets also note that these benchmarks show maximum parameters for grid performance. It doesnt mean that every application will operate in such stress environment. Besides, ");
            sb.Append("it is hard to imagine a situation when every cell in the visible part of grid (there may be over 500 of them) will be updated at a rate of over 10 updates per second. ");
            sb.Append("In real-life applications this value doesnt exceed 1-2 updates for 5-10 cells per second. Data provided below enables a developer to evaluate application performance range in ");
            sb.Append("the beginning of project and to start creating code knowing, which mode is the best to use our visual components.</p>");

            sb.Append("<p><b>Performance of data insertion into a grid</b></p>");
            sb.Append("<p>It is most efficient to insert elements in the end of the grid (this is true for all grids). This method of adding data basically doesnt depend on the number of already inserted elements. ");
            sb.Append("Adding data to the beginning of grid or to a sorted grid heavily depends on the number of already inserted elements. This is especially true for insertion to the beginning of a grid, ");
            sb.Append("which should be avoided.</p>");
            sb.AppendFormat("<p>Inserting data to the beginning and sorted grid: <img src=\"{0}inserting_beginning.png\"/></p>", prefix);
            sb.AppendFormat("<p>Inserting data to the end: <img src=\"{0}inserting_end.png\"/></p>", prefix);
            
            sb.Append("<p><b>Removing data</b></p>");
            sb.Append("<p>Data removal is also an important characteristic of a grid. Slow removal of data may cause application crashing, especially when they are closed. ");
            sb.Append("It is most efficient to remove data from the end of the grid. Data removal rate doesnt depend much on data volume. It is better to avoid data removal from the beginning of grid ");
            sb.Append("or from a sorted grid because these methods are efficient only for low data volumes. Hint: To speed up data removal, cancel sorting with <b>Header.SortedColumns.Clear()</b>.</p>");

            sb.AppendFormat("<p>Removing data from the beginning and sorted grid: <img src=\"{0}removing_beginning.png\"/></p>", prefix);
            sb.AppendFormat("<p>Removing data from the end: <img src=\"{0}removing_end.png\"/></p>", prefix);

            sb.Append("<p><p><b>Updating data in non-event model</b></p></p>");
            sb.AppendLine("<p>This model assumes that data doesnt implement <b>INotifyPropertyChanged</b> interface. Example:</p><pre>");

            sb.AppendLine("grid.Rows.Add(<span class=\"highlight-keyword\">new</span> <span class=\"highlight-keyword\">object</span>[]{<span class=\"highlight-literal\">\"value1\"</span>, <span class=\"highlight-number\">123</span>, <span class=\"highlight-keyword\">true</span>, DateTime.Now});");
            sb.AppendLine("<span class=\"highlight-comment\">//Data is updated by calling Cell.Value property.</span>");
            sb.AppendLine("Row row = ...;");
            sb.AppendLine("Cell cell = row[<span class=\"highlight-number\">0</span>] ;");
            sb.AppendLine("cell.Value = <span class=\"highlight-literal\">\"value2\"</span>;</pre>");

            sb.Append("<p>This property can be called from any thread, not just from GUI thread. In this case the grid synchronizes threads with synchronous call of <b>Control.Invoke()</b>.</p>");
            sb.Append("<p>Maximum data update rate heavily depends on data volume. When there are a lot of rows, updating speed increases because there are less updates in the visible area. When the number of ");
            sb.Append("rows increases as does overall updating rate, data synchronization between threads puts more load on CPU from non-GUI thread.");
            sb.AppendFormat("<img src=\"{0}rt_updating.png\"/></p>", prefix);


            sb.Append("<p>Its worth mentioning that fading effect, when every cell is repainted every 30 ms with color change, has a negative impact on performance and should not be used in real-time applications.");
            sb.AppendFormat("<img src=\"{0}updating_fading.png\"/></p>", prefix);
            sb.Append("<p>Real-time sorting speed also heavily depends on the number of rows and on vertical scrollbar position. In the middle of a grid sorting of one row requires repainting of many neighboring ");
            sb.Append("rows that also change their positions during rotation.");
            sb.AppendFormat("<img src=\"{0}rt_sorting.png\"/></p>", prefix);


            sb.Append("<p><b>Updating data in an event-driven model</b></p>");
            sb.Append("<p>This model enables working with data implementing INotifyPropertyChanged interface.</p>");
            sb.Append("<p>In general, grid characteristics in this mode are similar to non-event mode. The main distinction is the ability to synchronize data with asynchronous call that doesnt block the called thread. ");
            sb.Append("This synchronization method consumes more resources, however it is more thread-safe and helps to prevent thread deadlocks.</p>");
            sb.AppendFormat("<p><img src=\"{0}edm_rt_updating.png\"/></p>", prefix);
            sb.AppendFormat("<p><img src=\"{0}edm_fading.png\"/></p>", prefix);
            sb.AppendFormat("<p><img src=\"{0}edm_sorting.png\"/></p>", prefix);


            sb.Append("<p><b>Data grouping</b></p>");
            sb.Append("<p>Below we provide grid characteristics for static and real-time data grouping. During static grouping a grid rebuilds content for all data in the grid, while dependence of grouping ");
            sb.Append("rate on data volume is measured. During dynamic grouping data is changed in real-time, and this requires searching for existing groups, creating new groups and removing groups that contain ");
            sb.Append("no more rows. To understand the performance chart for dynamic data regrouping, remember that when there are a few rows, they have to be repainted frequently, and when there are many rows, ");
            sb.Append("more resources are consumed for searching or creating groups.</p>");
            sb.AppendFormat("<p><img src=\"{0}grouping.png\"/></p>", prefix);
            sb.AppendFormat("<p><img src=\"{0}rt_regrouping.png\"/></p>", prefix);

            sb.Append("<p><b>Dynamic filtering</b></p>");
            sb.Append("<p>Filtering manages row visibility. A developer may set his own filters to manage filtering. Every time when new data is added to a grid or when data is modified, the grid checks whether ");
            sb.Append("this data meets filtering conditions.</p> ");
            sb.Append("<p>Performance heavily depends on the number of rows and on vertical scrollbar position. When the thumb is in the middle, grid has to repaint more rows because when visibility status ");
            sb.Append("of a higher-standing row changes, all the following rows change their positions.");
            sb.AppendFormat("<img src=\"{0}rt_filtering.png\"/></p>", prefix);

            sb.Append("</body></html>");
            

            //Populate 'NetGrid Performance' tab
            webBrowser.DocumentText = sb.ToString();
        }

        private void OnTabDataUpdatingSelectedIndexChanged(object sender, EventArgs e)
        {
            addingRowsControl1.StopTest();
            removingRowsControl1.StopTest();
            dataUpdatingControl1.StopTest();
            dataGroupingControl1.StopTest();
            dataFilteringControl1.StopTest();

            
            logViewer1.Visible = tabDataUpdating.SelectedTab != tabNetGridPerformance;
        }
    }
}