using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Diagnostics;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Grid.Benchmark.RemovingRows
{
    public partial class RemovingRowsControl : UserControl
    {
        static readonly HighPerformanceTimer hiPerfTimer = new HighPerformanceTimer();
        private bool stopping;

        public RemovingRowsControl()
        {
            InitializeComponent();

            //Initialize header and columns
            grid.Headers.Add(Header.FromDataType(typeof(DataObject)));
            foreach (Column column in grid.Headers[0])
            {
                column.Width = 80;
            }

            grid.PaintRow += delegate(object sender, PaintRowEventArgs e)
            {
                e.Parts &= e.Parts ^ PaintPart.RowSelectorImages;

                e.PaintAll();
                e.Handled = true;

                using(StringFormat sf = new StringFormat())
                {
                    sf.Alignment = StringAlignment.Far;
                    e.Graphics.DrawString(e.Row.VisibleIndex.ToString(), SystemFonts.CaptionFont, SystemBrushes.InfoText, e.RowSelectorBounds, sf);    
                }
            };


            grid.PaintRectangle += delegate(object sender, PaintRectangleEventArgs e)
            {
                if (grid.Nodes.Count == 0)
                {
                    e.PaintAll();
                    e.Handled = true;

                    //Draw text and logo if there are no rows
                    Point pt = new Point(e.Grid.Width - Properties.Resources.logo.Width, e.Grid.Height - Properties.Resources.logo.Height);
                    e.Graphics.DrawImage(Properties.Resources.logo, new Rectangle(pt, Properties.Resources.logo.Size));

                    Rectangle rc = grid.ClientRectangle;
                    rc.Width -= Properties.Resources.logo.Width;
                    rc.Offset(10, 30);

                    using (Font f = new Font("verdana", 13, GraphicsUnit.Pixel))
                    using (Brush b = new SolidBrush(Color.FromArgb(0x55, 0x66, 0x55)))
                    using (StringFormat sf = new StringFormat())
                    {
                        sf.Alignment = StringAlignment.Near;
                        sf.LineAlignment = StringAlignment.Near;
                        sf.FormatFlags &= StringFormatFlags.NoWrap ^ sf.FormatFlags;
                        e.Graphics.DrawString("Removing data:\r\n\r\nData removal is also an important characteristic of a grid. Slow removal of data may cause application crashing, especially when they are closed. It is most efficient to remove data from the end of the grid. Data removal rate doesnt depend much on data volume. It is better to avoid data removal from the beginning of grid or from a sorted grid because these methods are efficient only for low data volumes.\r\n\r\nHint: To speed up data removal, cancel sorting with Header.SortedColumns.Clear()", f, b, rc, sf);
                    }
                }
            };

        }

        public void StopTest()
        {
            //Remove sort
            grid.Headers[0].SortedColumns.Clear();

            grid.Rows.Clear();

            stopping = true;
        }

        private void OnBtnStartClick(object sender, EventArgs e)
        {
            Random random = new Random();
            panel.Enabled = false;

            //Remove sort
            grid.Headers[0].SortedColumns.Clear();

            int rowCount = 500000;
            int tranche = 10000;

            //Limit amount of data
            if(rbFromEnd.Checked)
            {
                Logger.Info(string.Format("Performance measurement when rows are removed from the end of grid"));
                rowCount = 500000;
                tranche = 10000;
            }
            else if(rbFromBeginning.Checked)
            {
                Logger.Info(string.Format("Performance measurement when rows are removed from the beginning of grid"));
                rowCount = 100000;
                tranche = 10000;
            }
            else if (rbFromSortedGrid.Checked)
            {
                Logger.Info(string.Format("Performance measurement when rows are removed from sorted grid"));
                rowCount = 100000;
                tranche = 10000;
            }

            //Populate the grid
            stopping = false; 
            if (grid.Rows.Count == 0)
            {
                Logger.Info(string.Format("Prepare .Net Grid and populate it with {0} rows...", rowCount));
                Application.DoEvents();
                for (int i = 0; (i < rowCount) && !stopping; ++i)
                {
                    grid.Rows.Add(new DataObject());
                }
            }

            //Enable sort if needed
            if (rbFromSortedGrid.Checked)
            {
                Logger.Info(string.Format("Sort {0} rows...", rowCount));
                Application.DoEvents();
                grid.Headers[0][1].SortDirection = SortDirection.Ascending;
                grid.Headers[0][3].SortDirection = SortDirection.Descending;

                Logger.Info(string.Format("Now remove them from random positions..."));
                Application.DoEvents();
            }

            hiPerfTimer.Start();

            for (int i = grid.Rows.Count - 1; (i >= 0) && !stopping; --i)
            {
                if(rbFromEnd.Checked)
                {
                    //Remove from the end
                    grid.Rows[grid.Rows.Count - 1].Remove();    
                }
                else if (rbFromBeginning.Checked) 
                {
                    //From the beginning
                    grid.Rows[0].Remove();    
                }
                else 
                {
                    //Remove random row from the sorted grid
                    grid.Rows[random.Next() % grid.Rows.Count].Remove();
                }

                if (i % tranche == 0)
                {
                    hiPerfTimer.Stop();
                    

                    double duration = hiPerfTimer.Duration;

                    int rate = duration > 0 ? (int) (1000*tranche/duration) : 0;

                    if(rbFromEnd.Checked)
                    {
                        Logger.Info(string.Format("{2} rows ({0} - {1}) removed from the end in {3:0} msec. Rate: {4} rows/sec", i, i + tranche - 1, tranche, duration, rate));    
                    }
                    else if (rbFromBeginning.Checked)
                    {
                        Logger.Info(string.Format("Grid of {0} rows: first {1} rows removed from the beginning in {2:0} msec. Rate: {3} rows/sec", grid.Rows.Count + tranche,  tranche, duration, rate));    
                    }
                    else 
                    {
                        //Sorted grid
                        Logger.Info(string.Format("Grid of {0} rows: random {1} rows removed from the sorted grid in {2:0} msec. Rate: {3} rows/sec", grid.Rows.Count + tranche, tranche, duration, rate));
                    }
                    
                    Application.DoEvents();

                    hiPerfTimer.Start();
                }
            }

            hiPerfTimer.Stop();

            panel.Enabled = true;
            Logger.Info(string.Format("Measurement completed"));
        }
    }
}